SystemOrganization addCategory: #Proxies!

TestCase subclass: #ClassProxyInstVarTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies'!

!ClassProxyInstVarTest methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 10/3/2010 10:50'!
testClassSwap
	| inst proxy |
	inst := ClassWith1Var new.
	self assert: inst foo = 'foo'.
	proxy := ClassProxyInstVar new.
	proxy become: ClassWith1Var.
	self assert: (inst class == ClassWith1Var).
	self assert: inst foo = 'foo'.
! !

!ClassProxyInstVarTest methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 10/15/2010 18:27'!
testClassSwap2
	| inst proxy |
	inst := ClassWith1Var new.
	self assert: inst foo = 'foo'.
	self assert: inst class = ClassWith1Var.
	proxy := ClassProxyInstVar new.
	"I cannot do a becomeForward:   it has to be a become, since in the proxy I have an instVar pointing to self which will then updated to the class"
	proxy become: ClassWith1Var.
	
	"The following 2 lines freezes the image in case the Proxy extends from ProtoObject instead of Object"
"	self assert: inst class == proxy.
	Transcript show: ClassWith1Var name; cr."
	
	"All the following asserts are based in the assumption that it was a become: and not a becomeForward:"
	
	"ClassWith1Var class was replaced by an instance of ClassProxyInstVar"
	self assert: ClassWith1Var class name = #ClassProxyInstVar.
	self assert: inst class class name =  #ClassProxyInstVar.
	self assert: inst class class class isMeta.
	self assert: proxy class isMeta.
	self assert: proxy name = #ClassWith1Var.
	

	"With this #foo the proxy and the class are restored"
	self assert: inst foo = 'foo'.
	
	self assert: ClassWith1Var name = #ClassWith1Var.
	self assert: inst class class isMeta.
	self assert: inst class name =  #ClassWith1Var.
	self assert: proxy class name = #ClassProxyInstVar.

! !

TestCase subclass: #ObjectSwapperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies'!

!ObjectSwapperTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 9/30/2010 10:42'!
testAllNomralObjectsAllInner
	"A points to B which is an array that points to C and D. All are normal and inner objects"	
	| c d b a proxy fileName hello today initialInstances finalInstances |
	
	hello := 'Hello'.
	today := Date today.
	
	c := ClassWith1Var new var1: hello.
	d := ClassWith1Var new var1: today.
	b := Array with: c with: d.
	a := ClassWith1Var new var1: b.
	
	Smalltalk garbageCollect.
	initialInstances := ClassWith1Var instanceCount. 
	d := c := nil.
	
	"We put a proxy and become for B."
	ObjectSwapper new swap: b.
	
	Smalltalk garbageCollect.
	finalInstances := ClassWith1Var instanceCount. 
	"Since nobody else point to B after the become, D an C should be garbage collected"
	self assert: finalInstances = (initialInstances - 2). 
	

	self assert: a var1 = b.
	"With this, using the DNU the proxy will load back the objects"
	self assert: (a var1 first var1 = hello).
	self assert: (a var1 last var1 = today).
	self assert:  ClassWith1Var instanceCount = initialInstances.! !

!ObjectSwapperTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 9/30/2010 10:37'!
testAllNomralObjectsInnerAndShared
	"A points to B which is an array that points to C and D. All are normal and inner objects"	
	| c d b a proxy fileName hello today initialInstances finalInstances |
	
	hello := 'Hello'.
	today := Date today.
	
	c := ClassWith1Var new var1: hello.
	d := ClassWith1Var new var1: today.
	b := Array with: c with: d.
	a := ClassWith1Var new var1: b.
	
	Smalltalk garbageCollect.
	initialInstances := ClassWith1Var instanceCount. 
	d :=  nil.
	"We only nil D. C is still pointing, thus, it is a shared object."
	
	"We put a proxy and become for B."
	ObjectSwapper new swap: b.
	
	Smalltalk garbageCollect.
	finalInstances := ClassWith1Var instanceCount. 
	"Since nobody else point to B after the become, D (C will not) should be garbage collected"
	self assert: finalInstances = (initialInstances - 1). 
	

	self assert: a var1 = b.
	"With this, using the DNU the proxy will load back the objects"
	self assert: (a var1 first var1 = hello).
	self assert: (a var1 last var1 = today).
	"We cannot get duplicates of C."
	self assert:  ClassWith1Var instanceCount = initialInstances.
! !

!ObjectSwapperTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 9/30/2010 18:46'!
testOneClassObjectsInner
		
	| b a proxy |
	
	b := ClassWith2Var. 
	a := ClassWith1Var new var1: b.
	
	"We put a proxy and become for B."
	ObjectSwapper new swap: b.
	Transcript show: 'antes';cr.
	self assert: a var1 name = AClassProxy name. 
	Transcript show: 'desp';cr.
	a var1 new class.
	self assert: a var1 new class = ClassWith2Var.
	Transcript show: 'mas desp';cr.
	self assert: a var1 class = ClassWith2Var. 
	Transcript show: 'ree desp';cr.! !

Behavior subclass: #ClassProxy2
	instanceVariableNames: 'className'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies'!

!ClassProxy2 classSide methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 9/29/2010 14:04'!
aTest

| o p |
o := MyClass new.
o foo: 123.
p := ClassProxy2 new.
p become: MyClass.
MyClass flushCache.
ClassProxy2 flushCache.
o == nil.
Transcript show: o class name; cr.
o foo.
! !

!ClassProxy2 methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 9/29/2010 14:03'!
become: aClass
	className := aClass name.
	aClass fileOut.
	super become: aClass! !

!ClassProxy2 methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 9/29/2010 14:04'!
doesNotUnderstand: aMessage
	| reloadedClass |
	(FileStream fileNamed: className, '.st') fileIn.
	reloadedClass := Smalltalk at: className. 
	self becomeForward: reloadedClass.
	^aMessage sendTo: reloadedClass! !

Class subclass: #ClassProxy3
	instanceVariableNames: 'className'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies'!

!ClassProxy3 classSide methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 9/29/2010 14:18'!
aTest

| o p |
o := MyClass new.
o foo: 123.
p := ClassProxy3 new.
p become: MyClass.
MyClass flushCache.
ClassProxy3 flushCache.
o == nil.
Transcript show: o class name; cr.
o foo.
! !

!ClassProxy3 methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 9/29/2010 14:18'!
become: aClass
	className := aClass name.
	aClass fileOut.
	super become: aClass! !

!ClassProxy3 methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 9/29/2010 14:18'!
doesNotUnderstand: aMessage
	| reloadedClass |
	(FileStream fileNamed: className, '.st') fileIn.
	reloadedClass := Smalltalk at: className. 
	self becomeForward: reloadedClass.
	^aMessage sendTo: reloadedClass! !

ProtoObject subclass: #ClassProxy4
	instanceVariableNames: 'className'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies'!

!ClassProxy4 commentStamp: 'NouryBouraqadi 8/30/2010 11:15' prior: 0!
Evaluate the following. The last line causes the image to crash.

o := MyObject new.
o foo: 123.
p := ClassProxy new.
p become: MyObject.
o == nil.
o foo.
!

!ClassProxy4 classSide methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 9/30/2010 11:27'!
aTest

| o p |
o := MyClass new.
o foo: 123.
p := ClassProxy4 new.
p become: MyClass.
MyClass flushCache.
ClassProxy4 flushCache.
o == nil.
Transcript show: o class name; cr.
o foo.
! !

!ClassProxy4 methodsFor: 'accessing' stamp: 'NouryBouraqadi 8/30/2010 10:53'!
become: aClass
	className := aClass name.
	aClass fileOut.
	super become: aClass! !

!ClassProxy4 methodsFor: 'messagehandling' stamp: 'NouryBouraqadi 8/30/2010 11:13'!
doesNotUnderstand: aMessage
	| reloadedClass |
	(FileStream fileNamed: className, '.st') fileIn.
	reloadedClass := Smalltalk at: className. 
	self becomeForward: reloadedClass.
	^aMessage sendTo: reloadedClass! !

ProtoObject subclass: #ClassProxy5
	instanceVariableNames: 'superclass methodDict format'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies'!

!ClassProxy5 classSide methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 9/30/2010 16:27'!
aTest

| o p |
o := MyClass new.
o foo: 123.
p := ClassProxy5 new.
p become: MyClass.
MyClass flushCache.
ClassProxy5 flushCache.
o == nil.
Transcript show: o class name; cr.
o foo.
! !

!ClassProxy5 methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 9/30/2010 16:26'!
become: aClass
	"className := aClass name."
	aClass fileOut.
	super become: aClass! !

!ClassProxy5 methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 9/30/2010 16:25'!
cannotInterpret: aMessage
	Transcript show: 'somethiiing'; cr.! !

Object variableSubclass: #AClassProxy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies'!

AClassProxy class
	instanceVariableNames: 'fileName'!

AClassProxy class
	instanceVariableNames: 'fileName'!

!AClassProxy classSide methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 9/30/2010 18:10'!
fileName
	^ fileName ! !

!AClassProxy classSide methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 9/30/2010 16:42'!
fileName: aFileName
	fileName := aFileName! !

!AClassProxy classSide methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 9/30/2010 17:22'!
forFileName: aFileName
	
	| anonClass |
	
	anonClass := self shallowCopy.
	anonClass fileName: aFileName; methodDict: nil ; superclass: self.
	
	^ anonClass "basicNew"! !

!AClassProxy classSide methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 9/30/2010 18:10'!
handleMessage: aMessage forProxy: proxy
	
	self becomeForward: self loadBackSerializedObjectsFromFile.
	^ self perform: aMessage selector withArguments: aMessage arguments
! !

!AClassProxy classSide methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 9/30/2010 16:52'!
loadBackSerializedObjectsFromFile

	| file result |
	file := FileStream fileNamed: self fileName.
	file binary.
	result := (ReferenceStream on: file) next.
	file close.
	^ result! !

!AClassProxy methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 9/30/2010 18:04'!
cannotInterpret: aMessage
	Transcript show: 'cannotInterpret: ', aMessage selector asString; cr.
	^ aMessage lookupClass handleMessage: aMessage forProxy: self! !

Object subclass: #ClassProxyInstVar
	instanceVariableNames: 'superclass methodDict format actualClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies'!

!ClassProxyInstVar methodsFor: 'initialize-release' stamp: 'NouryBouraqadi 10/1/2010 14:57'!
initialize
	super initialize.
	superclass := ProxySuperclass.
	methodDict := nil.
	actualClass := self! !

!ClassProxyInstVar methodsFor: 'initialize-release' stamp: 'MarianoMartinezPeck 10/2/2010 19:28'!
restore
	Transcript show: 'Will restore class'; cr.
	self become: actualClass.
	Transcript show: 'Class restored'; cr.
! !

Object subclass: #ClassWith1Var
	instanceVariableNames: 'var1'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies'!

!ClassWith1Var methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 10/2/2010 19:26'!
foo
	^ 'foo'! !

!ClassWith1Var methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 9/29/2010 15:30'!
var1
	^ var1! !

!ClassWith1Var methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 9/29/2010 15:30'!
var1: anObject
	var1 := anObject! !

Object subclass: #ClassWith2Var
	instanceVariableNames: 'var1 var2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies'!

!ClassWith2Var methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 9/29/2010 15:30'!
var1
	^ var1! !

!ClassWith2Var methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 9/29/2010 15:30'!
var1: anObject
	var1 := anObject! !

!ClassWith2Var methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 9/29/2010 15:30'!
var2
	^ var2! !

!ClassWith2Var methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 9/29/2010 15:30'!
var2: anObject
	var2 := anObject! !

Object variableSubclass: #MessageCatchingProxy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies'!

MessageCatchingProxy class
	instanceVariableNames: 'unwrapSelector object'!

MessageCatchingProxy class
	instanceVariableNames: 'unwrapSelector object'!

!MessageCatchingProxy class methodsFor: 'as yet unclassified' stamp: 'sig 11/24/2008 23:56'!
handleMessage: aMessage forProxy: proxy

	self shouldBeImplemented ! !

!MessageCatchingProxy class methodsFor: 'as yet unclassified' stamp: 'sig 11/24/2008 23:55'!
setObject: anObject

	object := anObject! !

!MessageCatchingProxy class methodsFor: 'as yet unclassified' stamp: 'sig 11/24/2008 23:56'!
wrap: anObject

	| anonClass |
	
	anonClass := self clone.
	anonClass setObject: anObject; methodDict: nil ; superclass: self.
	
	^ anonClass basicNew! !

!MessageCatchingProxy methodsFor: 'as yet unclassified' stamp: 'sig 11/25/2008 00:03'!
cannotInterpret: aMessage
	^ aMessage lookupClass handleMessage: aMessage forProxy: self
! !

Object subclass: #ObjectProxy
	instanceVariableNames: 'fileName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies'!

!ObjectProxy classSide methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 9/29/2010 15:39'!
aTest

| o p |
o := MyClass new.
o foo: 123.
p := ClassProxy3 new.
p become: MyClass.
MyClass flushCache.
ClassProxy3 flushCache.
o == nil.
Transcript show: o class name; cr.
o foo.
! !

!ObjectProxy classSide methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 9/29/2010 16:51'!
forFileName: aFileName
	^ self new fileName: aFileName ! !

!ObjectProxy methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 9/29/2010 16:35'!
doesNotUnderstand: aMessage
	self becomeForward: self loadBackSerializedObjectsFromFile.
	^ self perform: aMessage selector withArguments: aMessage arguments! !

!ObjectProxy methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 9/29/2010 16:19'!
fileName
	^ fileName! !

!ObjectProxy methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 9/29/2010 16:19'!
fileName: anObject
	fileName := anObject! !

!ObjectProxy methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 9/29/2010 16:49'!
loadBackSerializedObjectsFromFile

	| file result |
	file := FileStream fileNamed: self fileName.
	file binary.
	result := (ReferenceStream on: file) next.
	file close.
	^ result! !

Object subclass: #ObjectSwapper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies'!

!ObjectSwapper methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 9/30/2010 10:03'!
serializeAndWriteToDisk: anObject	
	| file result |

	file := FileStream fileNamed: (self name, ' test').
	file binary.
	(ReferenceStream on: file) nextPut: anObject.
	file close.
	^ self name, ' test'.! !

!ObjectSwapper methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 9/30/2010 16:43'!
swap: anObject
	| fileName proxy |
	fileName := self serializeAndWriteToDisk: anObject. 
	
	"make this nicer"
	anObject isBehavior 
		ifTrue: [proxy := AClassProxy forFileName: fileName.]
		ifFalse: [proxy := ObjectProxy forFileName: fileName.].
	
	anObject becomeForward: proxy. ! !

Object subclass: #ProxySuperclass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Proxies'!

!ProxySuperclass methodsFor: 'swapping' stamp: 'NouryBouraqadi 10/1/2010 15:08'!
cannotInterpret: aMessage
	Transcript cr; 
		show: 'cannotInterpret:';
		space;
		show: aMessage selector..
	self class restore.
	^aMessage sendTo: self! !
